EventEmitter			= require "eventemitter3"
{bridge} 				= require "./Bridge"
{ViewerTools}			= require "./tools/ViewerTools"

BUILDS = 0

parseUrl = (url) ->
	# https://gist.github.com/jlong/2428561
	parser = document.createElement("a")
	parser.href = url
	return parser

class Runtime extends EventEmitter

	constructor: ->
		@setup()

	setup: ->

		# Listen for device changes and pass them on to the viewer so we
		# can update the menus in the toolbar
		if Framer?.Device
			properties = [ "deviceScale", "contentScale", "deviceType", 
				"keyboard", "orientation", "fullScreen"]
			properties.map (propertyName) ->
				Framer.Device.on "change:#{propertyName}", ->
					bridge.send("device:change")

			Framer.Device.on "change:phoneScale", (phoneScale) ->
				bridge.send("change:phoneScale", {phoneScale: phoneScale})

			Framer.Device._calculatePhoneScale()

		# Add a responder to the bridge, that provides layer properties:
		bridge.responders["layerProperties"] = (layerData) =>

			{ layerHash, layerState } = layerData

			layer = ViewerTools.getLayerByHash(layerHash)
			if not layer?
				console.log("WARNING: Layer #{layerHash} not found!")
				return null

			# Freeze the context, and switch the layer to the requested state:
			Framer.CurrentContext.freeze() unless Framer.CurrentContext.__framerViewFrozen
			Framer.CurrentContext.__framerViewFrozen = true

			# Switch to the requested state:
			layer.states.switchInstant(layerState)

			# Now send out the selected layer properties:
			result = {}

			for key, value of layer
				if (value instanceof Color)
					result[key] = value.toRgbString()
				else if not (_.isArray(value) or _.isObject(value) or _.isFunction(value) or isNaN(value))
					result[key] = value

			return result

		# Tell Framer Studio we are ready
		bridge.send("runtime.init")

		# Set up the error handler
		@_errorHandlerSetup()

	reset: ->
		Framer.CurrentContext.emit("willReset")
		Utils.reset()

	_errorHandler: (runtimeError) =>

		# This function catches runtime errors, so after the coffeescript was successfully
		# compiled without syntax errors. We have to detect if they come from the compiled
		# coffeescript or from any other library (most likely framer.js). The way to detect
		# it seems to be the .filename property, which is equal to the current location if
		# the error comes from the compiled coffeescript.

		errorFromCompiledCoffeeScript = (runtimeError.filename == window.location.href)

		error = 
			message: runtimeError.message
			lineNumber: runtimeError.lineno
			colNumber: runtimeError.colno
			fileName: runtimeError.filename

		if errorFromCompiledCoffeeScript
			error.type = "coffeescript"
			error.clientCodeId = window.__framerClientCodeId

		else
			fileName = _.last(parseUrl(runtimeError.filename).pathname.split("/"))
			error.type = "javascript"
			error.message = "[#{fileName}] #{runtimeError.message}"

		bridge.send("runtime.error", error)

	_errorHandlerSetup: ->
		window.addEventListener("error", @_errorHandler)

	_errorHandlerRemove: ->
		window.removeEventListener("error", @_errorHandler)

exports.runtime = new Runtime()
